Struct gmp_mpfr::Integer
[−]
[src]
pub struct Integer { /* fields omitted */ }Integer holds an arbitrary-precision integer.
Standard arithmetic operations, bitwise operations and comparisons
are supported. In standard arithmetic operations such as addition,
you can mix Integer and primitive integer types; the result will
be an Integer.
Internally the integer is not stored using two's-complement representation, however, for bitwise operations and shifts, the functionality is the same as if the representation was using two's complement.
Examples
use gmp_mpfr::Integer; let mut i = Integer::from(1) << 1000; // i is now 1000000... (1000 zeros) assert!(i.size_in_bits() == 1001); assert!(i.find_one(0) == Some(1000)); i -= 1; // i is now 111111... (1000 ones) assert!(i.count_ones() == Some(1000)); let a = Integer::from(0xf00d); let all_ones_xor_a = Integer::from(-1) ^ &a; // a is unchanged as we borrowed it let complement_a = !a; // now a has been moved, so this would cause an error: // assert!(a > 0); assert!(all_ones_xor_a == complement_a); assert!(complement_a == -0xf00e); assert!(format!("{:x}", complement_a) == "-f00e");
Methods
impl Integer[src]
fn new() -> Integer
Constructs a new arbitrary-precision integer with value 0.
fn to_u32(&self) -> u32
Converts to a u32.
If the value is too large for the target type,
only the least-significant bits are returned.
fn to_i32(&self) -> i32
Converts to an i32.
If the value is too large for the target type,
only the least-significant bits are returned.
fn to_f64(&self) -> f64
Converts to an f64 rounding towards zero.
fn to_f32(&self) -> f32
Converts to an f32 rounding towards zero.
fn div_rem(&mut self, divisor: &mut Integer)
Computes the quotient and remainder of self divided by
divisor. The remainder is stored indivisor`.
fn abs(&mut self) -> &mut Integer
Computes the absolute value of self.
fn div_exact(&mut self, other: &Integer) -> &mut Integer
Divides self by other. This is much faster than normal
division, but produces correct results only when the division
is exact.
Panics
Panics if other is zero.
fn is_divisible(&self, other: &Integer) -> bool
Returns true if self is divisible by other.
fn is_congruent(&self, c: &Integer, d: &Integer) -> bool
Returns true if self is congruent to c modulo d, that
is, if there exists a q such that self == c + q * d.
Unlike other division functions, d can be zero.
fn root(&mut self, n: u32) -> &mut Integer
Computes the nth root of self and truncates the result.
fn root_rem(&mut self, buf: &mut Integer, n: u32)
Computes the nth root of self and returns the truncated
root and the remainder. The remainder is self minus the
truncated root raised to the power of n.
The remainder is stored in buf.
fn sqrt(&mut self) -> &mut Integer
Computes the square root of self and truncates the result.
fn sqrt_rem(&mut self, buf: &mut Integer)
Computes the square root of self and returns the truncated
root and the remainder. The remainder is self minus the
truncated root squared.
The remainder is stored in buf.
fn is_perfect_power(&self) -> bool
Returns true if self is a perfect power.
fn is_perfect_square(&self) -> bool
Returns true if self is a perfect square.
fn gcd(&mut self, other: &Integer) -> &mut Integer
Finds the greatest common divisor. The result is always positive except when both inputs are zero.
fn lcm(&mut self, other: &Integer) -> &mut Integer
Finds the least common multiple. The result is always positive except when one or both inputs are zero.
fn invert(&mut self, m: &Integer) -> Option<&mut Integer>
fn set_factorial(&mut self, n: u32) -> &mut Integer
Computes the factorial of n.
The value of self is ignored.
fn set_factorial_2(&mut self, n: u32) -> &mut Integer
Computes the double factorial of n.
The value of self is ignored.
fn set_factorial_m(&mut self, n: u32, m: u32) -> &mut Integer
Computes the m-multi factorial of n.
The value of self is ignored.
fn set_primorial(&mut self, n: u32) -> &mut Integer
Computes the primorial of n.
The value of self is ignored.
fn binomial(&mut self, k: u32) -> &mut Integer
Computes the binomial coefficient self over k.
fn set_binomial(&mut self, n: u32, k: u32) -> &mut Integer
Computes the binomial coefficient n over k.
The value of self is ignored.
fn cmp_abs(&self, other: &Integer) -> Ordering
Compares the absolute values of self and other.
fn sign(&self) -> Ordering
Returns Less if self is less than zero,
Greater if self is greater than zero,
or Equal if self is equal to zero.
fn size_in_bits(&self) -> usize
Returns the number of bits required to represent the absolute
value of self.
fn count_ones(&self) -> Option<BitCount>
Returns the number of ones in self if the value >= 0.
fn ham_dist(&self, other: &Integer) -> Option<BitCount>
Retuns the Hamming distance between self and other if they
have the same sign, otherwise None.
fn find_zero(&self, start: BitCount) -> Option<BitCount>
Returns the location of the first zero, starting at start.
If the bit at location start is zero, returns start.
fn find_one(&self, start: BitCount) -> Option<BitCount>
Returns the location of the first one, starting at start.
If the bit at location start is one, returns start.
fn set_bit(&mut self, index: BitCount, val: bool) -> &mut Integer
Sets the bit at location index to 1 if val is true or 0
if val is false.
fn get_bit(&self, index: BitCount) -> bool
Returns true if the bit at location index is 1 or false
if the bit is 0.
fn invert_bit(&mut self, index: BitCount) -> &mut Integer
Toggles the bit at location index.
fn to_string_radix(&self, radix: i32) -> String
Returns a string representation of self for the specified
radix. If radix is > 36, 'a' to 'z' represent digits 10 to
36, and 'A' to 'Z' represent digits 37 to 62.
Panics
Panics if radix is less than 2 or greater than 62.